# Class to install a Mastodon instance
#
# [*ensure*]
#   String defining if is present or absent
#
# [*hostname*]
#   String with the full qualified hostname of the instance. There must be a DNS record for it.
#
# [*smtp_server*]
#   String with the SMTP server to use.
#
# [*mastodon_home*]
#   String path to Mastodon user home directory. Default /opt/mastodon
#
class mastodon (
  String $ensure = 'present',
  String $hostname = 'mastodon.example.org',
  String $smtp_server = 'mail.example.org',
  String $mastodon_home = '/opt/mastodon',
) {
  case $ensure {
    default: {
      $package_ensure = 'installed'
      $directory_ensure = 'directory'
      $link_ensure = 'link'
      $service_ensure = 'running'
      $file_ensure = 'present'
    }
    /^(absent|delete|uninstall|remove|unregister)$/: {
      $package_ensure = 'absent'
      $directory_ensure = 'absent'
      $link_ensure = 'absent'
      $file_ensure = 'absent'
      $service_ensure = 'stopped'
      $cron_ensure = 'absent'
    }
  }
  $packages = [
    'apt-transport-https',
    'autoconf',
    'bison',
    'build-essential',
    'ca-certificates',
    'certbot',
    'ffmpeg',
    'file',
    'g++',
    'gcc',
    'git-core',
    'gnupg',
    'imagemagick',
    'libffi-dev',
    'libgdbm-dev',
    'libicu-dev',
    'libidn11-dev',
    'libjemalloc-dev',
    'libncurses5-dev',
    'libpq-dev',
    'libprotobuf-dev',
    'libreadline6-dev',
    'libssl-dev',
    'libxml2-dev',
    'libxslt1-dev',
    'libyaml-dev',
    'lsb-release',
    'nginx',
    # 'nodejs',
    'pkg-config',
    # 'postgresql-contrib',
    # 'postgresql',
    'protobuf-compiler',
    'python3-certbot-nginx',
    # 'redis-server',
    # 'redis-tools',
    'wget',
    'zlib1g-dev',
  ]
  $packages.each | $package | {
    if (!defined(Package[$package])) {
      package { $package:
        ensure => $package_ensure,
      }
    }
  }
  class { 'nodejs':
    repo_url_suffix => '16.x',
  }
  class { 'postgresql::server':
  }
  include redis
  exec { 'enable_corepack':
    command => '/usr/bin/corepack enable',
    creates => '/usr/bin/yarn',
    require => Class['nodejs'],
  }
  exec { 'yarn_classic':
    command => '/usr/bin/yarn set version classic',
    creates => '/root/.yarnrc',
    require => Exec['enable_corepack'],
  }
  group { 'mastodon': }
  user { 'mastodon':
    gid        => 'mastodon',
    home       => $mastodon_home,
    managehome => true,
    system     => true,
    require    => Group['mastodon'],
  }
  vcsrepo { 'rbenv':
    path     => "${mastodon_home}/.rbenv",
    source   => 'https://github.com/rbenv/rbenv.git',
    provider => 'git',
    require  => User['mastodon'],
  }
  exec { 'configure_rbenv':
    command => "${mastodon_home}/.rbenv/src/configure",
    creates => "${mastodon_home}/.rbenv/src/Makefile",
  }
}
